'use strict';

require('mocha');
var assert = require('assert');
var bash = require('./');

describe('running extglob against minimatch tests', function() {
  var tests = [
    ['a/b/c', '**/c', true],
    ['a/b/c', '**/', false],
    ['a/b/c/', '**/', true],
    ['.a', '.*', true],
    ['*(a|b[)', '*(a|b\\[)', false],
    ['a', '*(a|b\\[)', true],
    ['b[', '*(a|b\\[)', true],
    ['123abc', 'ab*d+(e|f)', false],
    ['123abc', 'ab?*(e|f)', false],
    ['a', '!(a)', false],
    ['a', '@(a)', true],
    ['a', '@(b)', false],
    ['a', '*(a)', true],
    ['a', '+(a)', true],
    ['a', '?', true],
    ['a', '?(a|b)', true],
    ['a', '??', false],
    ['a', 'a?(a|b)', true],
    ['a', 'a?(x)', true],
    ['a', 'a??b', false],
    ['a', 'b?(a|b)', false],
    ['a.', '!(*.a|*.b|*.c)', true],
    ['a.', '*!(.a|.b|.c)', true],
    ['a.', '*.!(a)', true],
    ['a.', '*.!(a|b|c)', true],
    ['a.', '*.+(b|d)', false],
    ['a.a', '!(*.[a-b]*)', false],
    ['a.a', '!(*.a|*.b|*.c)', false],
    ['a.a', '!(*[a-b].[a-b]*)', false],
    ['a.a', '!*.(a|b)', false],
    ['a.a', '!*.(a|b)*', false],
    ['a.a', '@(a|d).(a|b)*', false],
    ['a.a', '*!(.a|.b|.c)', true],
    ['a.a', '*.!(a)', false],
    ['a.a', '*.+(b|d)', false],
    ['a.a.a', '!(*.[a-b]*)', false],
    ['a.a.a', '!(*[a-b].[a-b]*)', false],
    ['a.a.a', '!*.(a|b)', false],
    ['a.a.a', '!*.(a|b)*', false],
    ['a.a.a', '*.!(a)', true],
    ['a.a.a', '*.+(b|d)', false],
    ['a.abcd', '!(*.a|*.b|*.c)', true],
    ['a.abcd', '!(*.a|*.b|*.c)*', true],
    ['a.abcd', '*!(*.a|*.b|*.c)*', true],
    ['a.abcd', '*!(.a|.b|.c)', true],
    ['a.abcd', '*.!(a|b|c)', true],
    ['a.abcd', '*.!(a|b|c)*', true],
    ['a.b', '!(*.*)', false],
    ['a.b', '!(*.[a-b]*)', false],
    ['a.b', '!(*[a-b].[a-b]*)', false],
    ['a.b', '!*.(a|b)', false],
    ['a.b', '!*.(a|b)*', false],
    ['a.b', '@(a|d).(a|b)*', false],
    ['a.b', '*!(.a|.b|.c)', true],
    ['a.b', '*.!(a)', true],
    ['a.b', '*.+(b|d)', true],
    ['a.bb', '!(*.[a-b]*)', false],
    ['a.bb', '!(*[a-b].[a-b]*)', false],
    ['a.bb', '!*.(a|b)', false],
    ['a.bb', '!*.(a|b)*', false],
    ['a.bb', '@(a|d).(a|b)*', false],
    ['a.bb', '*.+(b|d)', true],
    ['a.c', '*!(.a|.b|.c)', true],
    ['a.c.d', '!(*.a|*.b|*.c)', true],
    ['a.c.d', '*!(.a|.b|.c)', true],
    ['a.c.d', '*.!(a|b|c)', true],
    ['a.ccc', '!(*.[a-b]*)', true],
    ['a.ccc', '!(*[a-b].[a-b]*)', true],
    ['a.ccc', '!*.(a|b)', false],
    ['a.ccc', '!*.(a|b)*', false],
    ['a.ccc', '*.+(b|d)', false],
    ['a.js', '!(*.js)', false],
    ['a.js', '*!(.js)', true],
    ['a.js', '*.!(js)', false],
    ['a.js.js', '!(*.js)', false],
    ['a.js.js', '*!(.js)', true],
    ['a.js.js', '*.!(js)', true],
    ['a.md', '!(*.js)', true],
    ['a.md', '*!(.js)', true],
    ['a.md', '*.!(js)', true],
    ['aa', '!(a)', true],
    ['aa', '?', false],
    ['aa', '@(a)b', false],
    ['aa', 'a??b', false],
    ['aaac', '*(@(a))a@(c)', true],
    ['aab', '?', false],
    ['aab', '??', false],
    ['aab', '@(c)b', false],
    ['aab', 'a??b', false],
    ['aaab', 'a??b', true],
    ['aac', '*(@(a))a@(c)', true],
    ['ab', '!(*.*)', true],
    ['ab', '@(a+|b)+', false],
    ['ab', 'a!(@(b|B))', false],
    ['aB', 'a!(@(b|B))', false],
    ['ab', 'ab**(e|f)', true],
    ['ab', 'ab**(e|f)g', false],
    ['ab', 'ab*+(e|f)', false],
    ['ab', 'ab*d+(e|f)', false],
    ['ab', 'ab?*(e|f)', false],
    ['ab]', 'a!(@(b|B))', true],
    ['abab', 'ab**(e|f)', true],
    ['abb', '!(*.*)', true],
    ['abbcd', '@(ab|a*(b))*(c)d', true],
    ['aBc', 'a!(@(b|B))', true],
    ['abcd', '?@(a|b)*@(c)d', true],
    ['abcd', '@(ab|a*@(b))*(c)d', true],
    ['abcdef', '@(a+|b)*', false],
    ['abcdef', '@(a+|b)+', false],
    ['abcdef', 'ab**(e|f)', true],
    ['abcdef', 'ab**(e|f)g', false],
    ['abcdef', 'ab*+(e|f)', true],
    ['abcdef', 'ab*d+(e|f)', true],
    ['abcdef', 'ab?*(e|f)', false],
    ['abcfef', '@(a+|b)*', false],
    ['abcfef', '@(a+|b)+', false],
    ['abcfef', 'ab**(e|f)', true],
    ['abcfef', 'ab**(e|f)g', false],
    ['abcfef', 'ab*+(e|f)', true],
    ['abcfef', 'ab*d+(e|f)', false],
    ['abcfef', 'ab?*(e|f)', true],
    ['abcfefg', '@(a+|b)*', false],
    ['abcfefg', '@(a+|b)+', false],
    ['abcfefg', 'ab**(e|f)', true],
    ['abcfefg', 'ab*d+(e|f)', false],
    ['abcfefg', 'ab?*(e|f)', false],
    ['abd', '@(a+|b)*', false],
    ['abd', '@(a+|b)+', false],
    ['abd', 'a!(@(b|B))', true],
    ['abd', 'a!(@(b|B))d', false],
    ['abd', 'ab**(e|f)', true],
    ['abd', 'ab*d+(e|f)', false],
    ['abd', 'ab?*(e|f)', true],
    ['abef', '@(a+|b)*', false],
    ['abef', '@(a+|b)+', false],
    ['abef', '*(a+|b)', false],
    ['abef', 'ab**(e|f)', true],
    ['abef', 'ab**(e|f)g', false],
    ['abef', 'ab*+(e|f)', true],
    ['abef', 'ab*d+(e|f)', false],
    ['abef', 'ab?*(e|f)', true],
    ['ac', '*(@(a))a@(c)', true],
    ['ac', 'a!(@(b|B))', true],
    ['acd', '@(a+|b)*', false],
    ['acd', '@(a+|b)+', false],
    ['acd', '@(ab|a*(b))*(c)d', true],
    ['acd', 'a!(@(b|B))', true],
    ['acd', 'a!(@(b|B))d', true],
    ['acd', 'ab*d+(e|f)', false],
    ['acd', 'ab?*(e|f)', false],
    ['ax', '?(a*|b)', true],
    ['ax', 'a?(b*)', false],
    ['b', '@(a+|b)*', true],
    ['b/a', '@(b/a)', true],
    ['b/b', '@(b/a)', false],
    ['b/c', '@(b/a)', false],
    ['ba', 'b?(a|b)', true],
    ['baaac', '*(@(a))a@(c)', false],
    ['bar', '@(foo)*', false],
    ['bar', '@(foo)b*', false],
    ['baz', '@(foo)*', false],
    ['baz', '@(foo)b*', false],
    ['bb', 'a?(a|b)', false],
    ['c', '*(@(a))a@(c)', false],
    ['c.a', '!(*.[a-b]*)', false],
    ['c.a', '!(*[a-b].[a-b]*)', true],
    ['c.a', '!*.(a|b)', false],
    ['c.a', '!*.(a|b)*', false],
    ['c.a', '*.!(a)', false],
    ['c.a', '*.+(b|d)', false],
    ['c.c', '*!(.a|.b|.c)', true],
    ['c.ccc', '!(*.[a-b]*)', true],
    ['c.ccc', '!(*[a-b].[a-b]*)', true],
    ['c.js', '!(*.js)', false],
    ['c.js', '*!(.js)', true],
    ['c.js', '*.!(js)', false],
    ['cow', '.!(*.*)', false],
    ['d.a.d', '!(*.[a-b]*)', false],
    ['d.a.d', '!(*[a-b].[a-b]*)', true],
    ['d.a.d', '!*.(a|b)', false],
    ['d.a.d', '!*.(a|b)*', false],
    ['d.a.d', '*.!(a)', true],
    ['d.a.d', '*.+(b|d)', true],
    ['d.d', '!(*.a|*.b|*.c)', true],
    ['d.d', '*!(.a|.b|.c)', true],
    ['d.d', '*.!(a|b|c)', true],
    ['d.js.d', '!(*.js)', true],
    ['d.js.d', '*!(.js)', true],
    ['d.js.d', '*.!(js)', true],
    ['e.e', '!(*.a|*.b|*.c)', true],
    ['e.e', '*!(.a|.b|.c)', true],
    ['e.e', '*.!(a|b|c)', true],
    ['effgz', '@(b+(c)d|e*(f)g?|?(h)i@(j|k))', true],
    ['efgz', '@(b+(c)d|e*(f)g?|?(h)i@(j|k))', true],
    ['egz', '@(b+(c)d|e*(f)g?|?(h)i@(j|k))', true],
    ['egz', '@(b+(c)d|e+(f)g?|?(h)i@(j|k))', false],
    ['egzefffgzbcdij', '*(b+(c)d|e*(f)g?|?(h)i@(j|k))', true],
    ['f', '!(f)', false],
    ['f', '*(!(f))', false],
    ['f', '+(!(f))', false],
    ['f.a', '!(*.a|*.b|*.c)', false],
    ['f.a', '*!(.a|.b|.c)', true],
    ['f.a', '*.!(a|b|c)', false],
    ['f.f', '!(*.a|*.b|*.c)', true],
    ['f.f', '*!(.a|.b|.c)', true],
    ['f.f', '*.!(a|b|c)', true],
    ['fff', '!(f)', true],
    ['fff', '*(!(f))', true],
    ['fff', '+(!(f))', true],
    ['fffooofoooooffoofffooofff', '*(*(f)*(o))', true],
    ['ffo', '*(f*(o))', true],
    ['fofo', '*(f*(o))', true],
    ['fofoofoofofoo', '*(fo|foo)', true],
    ['foo', '!(f)', true],
    ['foo', '!(foo)', false],
    ['foo', '!(foo)*', true],
    ['foo', '!(foo)+', false],
    ['foo', '!(foo)b*', false],
    ['foo', '!(x)', true],
    ['foo', '!(x)*', true],
    ['foo', '*(!(f))', true],
    ['foo', '*(!(foo))', true],
    ['foo', '+(!(f))', true],
    ['foo.js.js', '*.!(js)', true],
    ['foo.js.js', '*.!(js)*', true],
    ['foo.js.js', '*.!(js)*.!(js)', false],
    ['foo.js.js', '*.!(js)+', false],
    ['foob', '!(foo)b*', false],
    ['foobar', '!(foo)', true],
    ['foobar', '!(foo)*', true],
    ['foobar', '!(foo)b*', false],
    ['foobb', '!(foo)b*', true],
    ['foofoofo', '@(foo|f|fo)*(f|of+(o))', true],
    ['fooofoofofooo', '*(f*(o))', true],
    ['foooofo', '*(f*(o))', true],
    ['foooofof', '*(f*(o))', true],
    ['foooofof', '*(f+(o))', false],
    ['foooofofx', '*(f*(o))', false],
    ['foooxfooxfoxfooox', '*(f*(o)x)', true],
    ['foooxfooxfxfooox', '*(f*(o)x)', true],
    ['foooxfooxofoxfooox', '*(f*(o)x)', false],
    ['foot', '@(!(z*)|*x)', true],
    ['foox', '@(!(z*)|*x)', true],
    ['mad.moo.cow', '!(*.*).!(*.*)', false],
    ['mad.moo.cow', '.!(*.*)', false],
    ['Makefile', '!(*.c|*.h|Makefile.in|config*|README)', true],
    ['Makefile.in', '!(*.c|*.h|Makefile.in|config*|README)', false],
    ['moo', '.!(*.*)', false],
    ['moo.cow', '!(*.*).!(*.*)', true],
    ['moo.cow', '.!(*.*)', false],
    ['mucca.pazza', 'mu!(*(c))?.pa!(*(z))?', false],
    ['ofoofo', '*(of+(o))', true],
    ['ofoofo', '*(of+(o)|f)', true],
    ['ofooofoofofooo', '*(f*(o))', false],
    ['ofoooxoofxo', '*(*(of*(o)x)o)', true],
    ['ofoooxoofxoofoooxoofxo', '*(*(of*(o)x)o)', true],
    ['ofoooxoofxoofoooxoofxofo', '*(*(of*(o)x)o)', false],
    ['ofoooxoofxoofoooxoofxoo', '*(*(of*(o)x)o)', true],
    ['ofoooxoofxoofoooxoofxooofxofxo', '*(*(of*(o)x)o)', true],
    ['ofxoofxo', '*(*(of*(o)x)o)', true],
    ['oofooofo', '*(of|oof+(o))', true],
    ['ooo', '!(f)', true],
    ['ooo', '*(!(f))', true],
    ['ooo', '+(!(f))', true],
    ['oxfoxfox', '*(oxf+(ox))', false],
    ['oxfoxoxfox', '*(oxf+(ox))', true],
    ['parse.y', '!(*.c|*.h|Makefile.in|config*|README)', true],
    ['shell.c', '!(*.c|*.h|Makefile.in|config*|README)', false],
    ['xfoooofof', '*(f*(o))', false],
    ['zoot', '@(!(z*)|*x)', false],
    ['zoox', '@(!(z*)|*x)', true],
    ['zz', '@(a+|b)*', false],
  ];

  tests.forEach(function(test) {
    var fixture = test[0];
    var pattern = test[1];
    // if (pattern !== '*(a|b\\[)') return;
    var expected = test[2];
    var msg = 'should ' + (expected ? '' : 'not ')
      + 'match "' + fixture + '" with "' + pattern + '"';

    it(msg, function(cb) {
      try {
        var match = bash.isMatch(fixture, pattern);
      } catch (err) {
        cb(err);
        return;
      }
      assert.equal(match, expected, msg);
      cb();
    });
  });
});
